#######################################################################
# Copyright (c) 2025 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

require 'json'

module DruidQueryLogger
  def debug(message = nil, &block)
    if message.present? && message.include?('DRUID')
      json_part = extract_json(message)
      save_to_json_file(json_part) if json_part
    end
    super
  end

  private

  MUTEX = Mutex.new

  def extract_json(message)
    json_match = message.match(/\{.*\}/m)
    json_match ? json_match[0] : nil
  end

  def save_to_json_file(json_part)
    file_path = Rails.root.join('log', 'druid_query_logging.json')

    begin
      existing_data = File.exist?(file_path) ? JSON.parse(File.read(file_path)) : []
    rescue JSON::ParserError
      return # Skip this log entry instead of resetting
    end

    begin
      log_entry = JSON.parse(json_part)
      log_entry = log_entry.is_a?(Hash) ? log_entry.merge(
        'user' => safe_current_user_name,
        'creation_date' => Time.now
      ) : log_entry
    rescue JSON::ParserError
      return # Skip invalid JSON message
    end

    # Synchronize writing to the file using Mutex
    MUTEX.synchronize do
      existing_data << log_entry

      File.open(file_path, 'w') do |f|
        f.write(JSON.pretty_generate(existing_data))
      end
    end
  end

  def safe_current_user_name
    User.current_user.name || 'Unknown user'
  rescue NoMethodError
    'Unknown user'
  end
end

Rails.logger.singleton_class.prepend(DruidQueryLogger)
